    /**
    * @file BrushlessPIC.c
    * @brief This module contains the implementation of the brushless hardware
    * functions.

    * @author Stefan Popovic
    *
    * @date 11/09/2016
    *
    * @version 0.1a
    *
    * @copyright GNU Public License
    *
    * Test configuration:
    *
    * MCU: PIC18F45K22
    * Dev.Board: EasyPIC v7 Connectivity
    * Oscillator: HS-PLL 72.0000 MHz, internal 8.0000 MHz RC
    * Ext. Modules: Brushless Click Board
    * SW: MikroC PRO for PIC v6.6.3
    
    * NOTES:
      - Place Brushless click board on mikroBUS slot 2.
      - Turn ON the PORTC LEDs at SW3.3.
      - Put button jumper (J17) into VCC position and pull-down PORTD.
      - PWM: RC1
      - Vsense: RA3
      - Int: PB1

    /**************************************************************************
    * Module Preprocessor Constants
    **************************************************************************/
    #define CW    0x01
    #define CCW   0x00
    #define STEP 5

    /**************************************************************************
    * Module Preprocessor Macros
    **************************************************************************/


    /**************************************************************************
    * Module Typedefs
    **************************************************************************/


    /**************************************************************************
    * Module Variable Definitions
    **************************************************************************/
    sbit MOTOR_DIR at RE1_bit;

    unsigned int current_speed;
    unsigned int pwm_period;
    char txt[24];

    // Lcd module connections
    sbit LCD_RS at LATB4_bit;
    sbit LCD_EN at LATB5_bit;
    sbit LCD_D4 at LATB0_bit;
    sbit LCD_D5 at LATB1_bit;
    sbit LCD_D6 at LATB2_bit;
    sbit LCD_D7 at LATB3_bit;

    sbit LCD_RS_Direction at TRISB4_bit;
    sbit LCD_EN_Direction at TRISB5_bit;
    sbit LCD_D4_Direction at TRISB0_bit;
    sbit LCD_D5_Direction at TRISB1_bit;
    sbit LCD_D6_Direction at TRISB2_bit;
    sbit LCD_D7_Direction at TRISB3_bit;
    // End Lcd module connections

    char txt1[] = "PWM Duty";
    char txt2[] = "PWM MAX";


    // Lcd Initialization
    void Display_Init()
    {

      Lcd_Init();                        // Initialize Lcd
      Lcd_Cmd(_LCD_CLEAR);               // Clear display
      Lcd_Cmd(_LCD_CURSOR_OFF);          // Cursor off
      Lcd_Out(1, 1, txt1);
      Lcd_Out(2, 1, txt2);
      WordToStr(pwm_period, txt);
      Lcd_Out(2, 10, txt);

    }

    // MCU Initialization
    void MCU_Init()
    {

      ANSELA = 0;                         // Configure AN pins as digital
      ANSELB = 0;
      ANSELC = 0;
      ANSELD = 0;

      PORTD = 255;
      TRISD = 255;

      TRISE2_bit = 0;
      TRISC1_bit = 0;

      ADC_Init();                          // ADC Init
      Delay_ms(100);
      current_speed = 0;                  //initial value for current duty.
      pwm_period = 5000;

      PWM2_Init(5000);                    // Initialize PWM2 module at 5kHz
      PWM2_Set_Duty(current_speed);            // Set current duty for PWM2
      PWM2_Start();

    }

    void main()
    {
    
      MCU_Init();
      Display_Init();
      MOTOR_DIR = CCW;             // Setting motor direction to counter-clock-wise

        while(1)
        {

           if (RD0_bit)  // button on RD0 pressed
           {
                  Delay_ms(30);
                  current_speed = current_speed + STEP;     // increment current duty

                  if (current_speed > pwm_period)       // if we increase current duty greater then possible pwm_period value
                  {
                    current_speed = 0;                // reset current duty value to zero
                  }

                  PWM2_Set_Duty(current_speed);        // set newly acquired duty ratio
          }

            if (RD1_bit) {  // button on RD1 pressed
            
                  Delay_ms(30);
                  current_speed = current_speed - STEP;        // decrement current duty

                  if (current_speed > pwm_period) {      // if we increase current duty greater then possible pwm_period value
                  
                    current_speed = pwm_period;         // set current duty value to pwm_period
                  }

                  PWM2_Set_Duty(current_speed);        // set newly acquired duty ratio
             }



          Delay_ms(20);

          WordToStr(current_speed, txt);
          Lcd_Out(1, 10, txt);

       }
    }
